--
-- Cover (modified)
--
-- @FS17 author  Stefan Geiger
-- @to FS15 edited by agRush2k
-- @date  16/07/2014
-- @edit  20/10/2017
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

source("dataS/scripts/vehicles/specializations/SetCoverStateEvent.lua");

---Class for vehicles with a cover
-- @category Specializations
CoverModified = {};

---Checks if all prerequisite specializations are loaded
-- @param table specializations specializations
-- @return boolean hasPrerequisite true if all prerequisite specializations are loaded
-- @includeCode
function CoverModified.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end

---Called before loading
-- @param table savegame savegame
-- @includeCode
function CoverModified:preLoad(xmlFile)
    self.getIsToggleCoverAllowed = Cover.getIsToggleCoverAllowed;
    self.allowFillType = Utils.overwrittenFunction(self.allowFillType, CoverModified.allowFillType);
    self.getAllowFillFromAir = Utils.overwrittenFunction(self.getAllowFillFromAir, CoverModified.getAllowFillFromAir);
    self.setCoverState = Cover.setCoverState;
end

---Called on loading
-- @param table savegame savegame
-- @includeCode
function CoverModified:load(xmlFile)
    self.addFillTrigger = Utils.overwrittenFunction(self.addFillTrigger, CoverModified.addFillTrigger);
    self.removeFillTrigger = Utils.overwrittenFunction(self.removeFillTrigger, CoverModified.removeFillTrigger);

    self.coverAnimation = getXMLString(xmlFile, "vehicle.coverModified#animationName");
    if self.coverAnimation ~= nil then
        local isOpen = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.coverModified#openOnBuy"), false);
        if savegame ~= nil then
            isOpen = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#isCoverOpen"), isOpen);
        end
        self:setCoverState(isOpen, true);

        self.forceOpenOnTip = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.coverModified#forceOpenOnTip"), true);
        self.autoReactToTrigger = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.coverModified#autoReactToTrigger"), false);
    else
        self.isCoverOpen = true;
    end
end

---Returns attributes and nodes to save
-- @param table nodeIdent node ident
-- @return string attributes attributes
-- @return string nodes nodes
-- @includeCode
function CoverModified:getSaveAttributesAndNodes(nodeIdent)
    if self.coverAnimation ~= nil then
        local attributes = 'isCoverOpen="'..tostring(self.isCoverOpen)..'"';
        return attributes, nil;
    end
    return nil;
end

function CoverModified:delete()
end

function CoverModified:mouseEvent(posX, posY, isDown, isUp, button)
end

function CoverModified:keyEvent(unicode, sym, modifier, isDown)
end

---Called on client side on join
-- @param integer streamId streamId
-- @param integer connection connection
-- @includeCode
function CoverModified:readStream(streamId, connection)
    if connection:getIsServer() then
        if self.coverAnimation ~= nil then
            local isOpen = streamReadBool(streamId);
            self:setCoverState(isOpen, true);
        end
    end
end

---Called on server side on join
-- @param integer streamId streamId
-- @param integer connection connection
-- @includeCode
function CoverModified:writeStream(streamId, connection)
    if not connection:getIsServer() then
        if self.coverAnimation ~= nil then
            streamWriteBool(streamId, self.isCoverOpen);
        end
    end
end

---Called on update
-- @param float dt time since last call in ms
-- @includeCode
function CoverModified:update(dt)
    if self:getIsActiveForInput() then
        if self.coverAnimation ~= nil then
            if InputBinding.hasEvent(InputBinding.TOGGLE_COVER) then
                if self:getIsToggleCoverAllowed(not self.isCoverOpen) then
                    self:setCoverState(not self.isCoverOpen);
                end
            end
        end
    end
end

---Called on draw
-- @includeCode
function CoverModified:draw()
    if self.isClient and self:getIsActiveForInput(true) then
        if self.coverAnimation ~= nil and self:getIsToggleCoverAllowed(not self.isCoverOpen) then
            if self.isCoverOpen then
                g_currentMission:addHelpButtonText(g_i18n:getText("CLOSE_COVER"), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL);
            else
                g_currentMission:addHelpButtonText(g_i18n:getText("OPEN_COVER"), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL);
            end
        end
    end
--    if self.isServer then
--        if self.autoReactToTrigger then
--            if shouldOpen ~= self.isCoverOpen then
--                self:setCoverState(shouldOpen);
--            end
--        end
--    end
end

---Returns if toggle cover is allowed
-- @param boolean isCoverOpen is cover open
-- @return boolean allow allows cover toggle
-- @includeCode
function CoverModified:getIsToggleCoverAllowed(isCoverOpen)
    if self.tipState ~= nil then
        return self.tipState == Trailer.TIPSTATE_CLOSED;
    end
    return true;
end

---Returns if fill type is allowed (overwritten to block if cover is closed)
-- @param integer fillType fill type
-- @param boolean allowEmptying allow emptying
-- @return boolean allow allow fill type
-- @includeCode
function CoverModified:allowFillType(superFunc, fillType, allowEmptying)
    if not self.isCoverOpen then
        return false;
    end

    if superFunc ~= nil then
        return superFunc(self, fillType, allowEmptying);
    end

    return true;
end

---Set cover state
-- @param boolean isOpen new is open state
-- @param boolean noEventSend no event send
-- @includeCode
function CoverModified:setCoverState(isOpen, noEventSend)
    if self.coverAnimation ~= nil then
        SetCoverStateEvent.sendEvent(self, isOpen, noEventSend);
        self.isCoverOpen = isOpen;

        if isOpen then
            self:playAnimation(self.coverAnimation, -1, self:getAnimationTime(self.coverAnimation), true);
        else
            self:playAnimation(self.coverAnimation, 1, self:getAnimationTime(self.coverAnimation), true);
        end
    end
end

---Returns if fill from air is allowed
-- @return boolean isAllowed fill from air is allowed
-- @includeCode
function CoverModified:getAllowFillFromAir(superFunc)
    if not self.isCoverOpen then
        return false;
    end

    if superFunc ~= nil then
        return superFunc(self);
    end

    return true
end

---Called if vehicle is in fill trigger
-- @param table trigger trigger
-- @includeCode
function CoverModified:addFillTrigger(superFunc, trigger)
    if superFunc ~= nil then
        superFunc(self, trigger);
    end
    if self.fillTriggers ~= nil and table.getn(self.fillTriggers) > 0 then
        self:setCoverState(true);
    end
end

---Called if vehicle leaves fill trigger
-- @param table trigger trigger
-- @includeCode
function CoverModified:removeFillTrigger(superFunc, trigger)
    if superFunc ~= nil then
        superFunc(self, trigger);
    end
    if self.fillTriggers ~= nil and table.getn(self.fillTriggers) == 0 then
        self:setCoverState(false);
    end
end

---Called on start tipping
-- @param table tipTrigger tip trigger
-- @param integer tipReferencePointIndex index of tip reference point
-- @param boolean noEventSend no event send
-- @includeCode
function CoverModified:onStartTip(tipTrigger, tipReferencePointIndex, noEventSend)
    if self.coverAnimation ~= nil then
        if self.forceOpenOnTip then
            self:setCoverState(true, true);
        end
    end
end
